package transport

import (
	"net/http"

	"github.com/labstack/echo"
	"gitlab.informatika.org/k-01-11/graduit-be/src/module/pendaftaran/config"
	"gitlab.informatika.org/k-01-11/graduit-be/src/module/pendaftaran/entity"
	"gitlab.informatika.org/k-01-11/graduit-be/src/module/pendaftaran/internal/repository"
	"gitlab.informatika.org/k-01-11/graduit-be/src/module/pendaftaran/internal/usecase"
	"gitlab.informatika.org/k-01-11/graduit-be/src/utils"
)

type AdminPendaftaranHandler struct {
	pendaftaranUsecase usecase.PendaftaranUsecase
}

func NewAdminPendaftaranHandler(cfg config.PendaftaranTransportConfig) *AdminPendaftaranHandler {
	pendaftaranRepository := repository.NewPendaftaranRepository(cfg.DBWrite, cfg.DBRead)
	pendaftaranUsecase := usecase.NewPendaftaranUseCase(pendaftaranRepository)

	return &AdminPendaftaranHandler{
		pendaftaranUsecase: pendaftaranUsecase,
	}

}

func (t *AdminPendaftaranHandler) MountAdmin(group *echo.Group) {
	group.POST("/pendaftaran", t.AddPendaftaran)
	group.GET("/pendaftaran-get", t.GetPendaftaran)
	group.GET("/pendaftaran-get-by-dosen", t.GetPendaftaranByIdDosen)
	group.GET("/pendaftaran-delete", t.DeletePendaftaran)
	group.POST("/pendaftaran-update-status", t.UpdateStatusPendaftaran)
	group.POST("/pendaftaran-update-interview", t.UpdateWaktuInterview)
}

func (t *AdminPendaftaranHandler) AddPendaftaran(c echo.Context) error {
	request := new(entity.PendaftaranAddRequest)
	if err := c.Bind(&request); err != nil {
		return c.JSON(http.StatusBadRequest, utils.ResponseDetailOutput(false, http.StatusBadRequest, err.Error(), nil))
	}

	target := &entity.Pendaftaran{
		IdTopik:     request.IdTopik,
		IdMahasiswa: request.IdMahasiswa,
		IdDosen:     request.IdDosen,
	}

	newPendaftaran, err := t.pendaftaranUsecase.AddPendaftaran(*target)
	if err != nil {
		return c.JSON(http.StatusInternalServerError, utils.ResponseDetailOutput(false, http.StatusInternalServerError, err.Error(), nil))
	}

	return c.JSON(http.StatusOK, utils.ResponseDetailOutput(true, http.StatusCreated, "Pendaftaran added successfully", newPendaftaran))
}

func (t *AdminPendaftaranHandler) GetPendaftaran(c echo.Context) error {
	listGroupLimit, err := t.pendaftaranUsecase.GetPendaftaran()
	if err != nil {
		return c.JSON(http.StatusBadRequest, utils.ResponseDetailOutput(false, http.StatusBadRequest, err.Error(), nil))
	}

	return c.JSON(http.StatusOK, utils.ResponseDetailOutput(true, http.StatusOK, "Pendaftaran successfully retreived", listGroupLimit))
}

func (t *AdminPendaftaranHandler) GetPendaftaranByIdDosen(c echo.Context) error {
	id := c.QueryParam("id")
	listGroupLimit, err := t.pendaftaranUsecase.GetPendaftaranByIdDosen(id)
	if err != nil {
		return c.JSON(http.StatusBadRequest, utils.ResponseDetailOutput(false, http.StatusBadRequest, err.Error(), nil))
	}

	return c.JSON(http.StatusOK, utils.ResponseDetailOutput(true, http.StatusOK, "Pendaftaran successfully retreived", listGroupLimit))
}

func (t *AdminPendaftaranHandler) DeletePendaftaran(c echo.Context) error {
	id := c.QueryParam("id")
	listGroupLimit, err := t.pendaftaranUsecase.DeletePendaftaran(id)
	if err != nil {
		return c.JSON(http.StatusBadRequest, utils.ResponseDetailOutput(false, http.StatusBadRequest, err.Error(), nil))
	}

	return c.JSON(http.StatusOK, utils.ResponseDetailOutput(true, http.StatusOK, "Pendaftaran successfully deleted", listGroupLimit))
}

func (t *AdminPendaftaranHandler) UpdateStatusPendaftaran(c echo.Context) error {
	request := new(entity.UpdatePendaftaranRequest)
	if err := c.Bind(&request); err != nil {
		return c.JSON(http.StatusBadRequest, utils.ResponseDetailOutput(false, http.StatusBadRequest, err.Error(), nil))
	}

	newPendaftaran, err := t.pendaftaranUsecase.UpdateStatusPendaftaran(request.Id, request.Status)
	if err != nil {
		return c.JSON(http.StatusInternalServerError, utils.ResponseDetailOutput(false, http.StatusInternalServerError, err.Error(), nil))
	}

	return c.JSON(http.StatusOK, utils.ResponseDetailOutput(true, http.StatusCreated, "Pendaftaran status updated successfully", newPendaftaran))
}

func (t *AdminPendaftaranHandler) UpdateWaktuInterview(c echo.Context) error {
	request := new(entity.WaktuInterviewUpdateRequest)
	if err := c.Bind(&request); err != nil {
		return c.JSON(http.StatusBadRequest, utils.ResponseDetailOutput(false, http.StatusBadRequest, err.Error(), nil))
	}

	newPendaftaran, err := t.pendaftaranUsecase.UpdateWaktuInterview(request.Id, request.InterviewAt)
	if err != nil {
		return c.JSON(http.StatusInternalServerError, utils.ResponseDetailOutput(false, http.StatusInternalServerError, err.Error(), nil))
	}

	return c.JSON(http.StatusOK, utils.ResponseDetailOutput(true, http.StatusCreated, "Pendaftaran interview time updated successfully", newPendaftaran))
}
