package transport

import (
	"net/http"

	"github.com/labstack/echo"
	"gitlab.informatika.org/k-01-11/graduit-be/src/module/topic_allocation/entity"
	"gitlab.informatika.org/k-01-11/graduit-be/src/module/topic_allocation/config"
	"gitlab.informatika.org/k-01-11/graduit-be/src/module/topic_allocation/internal/repository"
	"gitlab.informatika.org/k-01-11/graduit-be/src/module/topic_allocation/internal/usecase"
	"gitlab.informatika.org/k-01-11/graduit-be/src/utils"
)

type AdminTopicAllocationHandler struct {
	topicUsecase usecase.TopicUsecase
}

func NewAdminTopicAllocationHandler(cfg config.TopicAllocationTransportConfig) *AdminTopicAllocationHandler {
	topicAllocationRepository := repository.NewTopicAllocationRepository(cfg.DBWrite, cfg.DBRead)
	topicUseCase := usecase.NewTopicUc(topicAllocationRepository)

	return &AdminTopicAllocationHandler{
		topicUsecase: topicUseCase,
	}
}

func (admin *AdminTopicAllocationHandler) MountAdmin(group *echo.Group) {
	group.GET("/alokasi-topik/:id", admin.GetTopicByID)
	group.GET("/alokasi-topik", admin.GetTopics)
	group.POST("/alokasi-topik", admin.AddTopic)
	group.PUT("/alokasi-topik/:id", admin.UpdateTopic)
	group.DELETE("/alokasi-topik/:id", admin.DeleteTopic)
}

func (admin *AdminTopicAllocationHandler) GetTopics(c echo.Context) error {
	param, err := utils.GetPageLimitSearchID(c.QueryParams())
	
	if err != nil {
		return c.JSON(http.StatusInternalServerError, utils.ResponseDetailOutput(false, http.StatusInternalServerError, err.Error(), nil))
	}

	list, err := admin.topicUsecase.GetTopic(param)
	if err != nil {
		return c.JSON(http.StatusBadRequest, utils.ResponseDetailOutput(false, http.StatusBadRequest, err.Error(), nil))
	}

	return c.JSON(http.StatusOK, utils.ResponseDetailOutput(true, http.StatusOK, "Topics retreived", list))
}

func (admin *AdminTopicAllocationHandler) GetTopicByID(c echo.Context) error {
	id := c.Param("id")

	if id != "" {
		item, err := admin.topicUsecase.GetTopicByID(id)

		if err != nil {
			return c.JSON(http.StatusBadRequest, utils.ResponseDetailOutput(false, http.StatusBadRequest, err.Error(), nil))
		}

		return c.JSON(http.StatusOK, utils.ResponseDetailOutput(true, http.StatusOK, "Topics retreived", item))
	} else {
		return c.JSON(http.StatusBadRequest, utils.ResponseDetailOutput(false, http.StatusBadRequest, "Incorrect URL", nil))
	}
}

func (admin *AdminTopicAllocationHandler) AddTopic(c echo.Context) error {
	topic := new(entity.Topic)

	if err := c.Bind(topic); err != nil {
		return c.JSON(http.StatusBadRequest, utils.ResponseDetailOutput(false, http.StatusBadRequest, err.Error(), nil))
	}

	newTopic, err := admin.topicUsecase.AddTopic(*topic)
	if err != nil {
		return c.JSON(http.StatusInternalServerError, utils.ResponseDetailOutput(false, http.StatusInternalServerError, err.Error(), nil))
	}

	return c.JSON(http.StatusOK, utils.ResponseDetailOutput(true, http.StatusOK, "Topic added successfully", newTopic))
}

func (admin *AdminTopicAllocationHandler) UpdateTopic(c echo.Context) error {
	id := c.Param("id")

	if id != "" {
		topic := new(entity.Topic)
		c.Bind(topic)
		topic.ID = id

		newItem, err := admin.topicUsecase.UpdateTopic(*topic)

		if err != nil {
			return c.JSON(http.StatusInternalServerError, utils.ResponseDetailOutput(false, http.StatusInternalServerError, "Failed to update topic", nil))
		}

		return c.JSON(http.StatusOK, utils.ResponseDetailOutput(true, http.StatusOK, "Topic updated", newItem))
	} else {
		return c.JSON(http.StatusBadRequest, utils.ResponseDetailOutput(false, http.StatusBadRequest, "Please provide topic id", nil))
	}
}

func (admin *AdminTopicAllocationHandler) DeleteTopic(c echo.Context) error {
	id := c.Param("id")

	if id != "" {
		err := admin.topicUsecase.DeleteTopic(id)

		if err != nil {
			return c.JSON(http.StatusInternalServerError, utils.ResponseDetailOutput(false, http.StatusInternalServerError, "Failed to delete topic", nil))
		}

		return c.JSON(http.StatusOK, utils.ResponseDetailOutput(true, http.StatusOK, "Topic deleted successfully", nil))
	} else {
		return c.JSON(http.StatusBadRequest, utils.ResponseDetailOutput(false, http.StatusBadRequest, "Please provide topic id", nil))
	}
}